function multiview(outfname, slices, images, options, points)

% MULTIVIEW  Display images with multiple overlays.
%
% MULTIVIEW(SLICES, IMAGES, [OPTIONS], [POINTS]) displays a background
% anatomical image with any number of functional overlays as well as other
% annotations. It is intended for the creation of publication-quality
% figures.
%
% SLICES is used to specify which slices to display. Each slice is
% specified in terms of whether it is axial, sagittal or coronal, what its
% coordinate is, and how it is positioned relative to other slices which
% have already been specified. SLICES is an array with the following
% fields, all of which are required:
%
%   PLANE  'h', 'a' or 'z' = axial
%          's' or 'x'      = sagittal
%          'c' or 'y'      = coronal
%   COORD  slice coordinate relative to the origin. See below for notes on
%          interpretation of location/orientation information in image
%          headers.
%   POS    'b' (below) or 'r' (to the right of) reference slice
%   REF    reference slice, i.e. which slice to position relative to
%
% The auxiliary program GETSLICES provides a shorthand for defining arrays
% of slices. type 'help getslices' for more information.
%
% IMAGES is an array of images to display. The first is the anatomical
% underlay, for which only the filename needs to be provided.
%
% Each subsequent element specifies a functional image. There are three
% different ways that functional images can be displayed: (1) standard
% overlays; (2) outlines, which are useful when multiple functional maps
% need to be simultaneously depicted); (3) blends, whereby colors are
% blended on the HSV color wheel with other functional images.
%
% Each functional element should have the following fields. Some are
% optional, denoted by [ ].
%
%   FNAME       filename
%   SCALE       2-element vector [min, max] values to map to color scale
%   CMAP        color scale; size should be 1 * 3 for a single color,
%                 N * 3 for a real color scale, or the name of a MATLAB
%                 colormap function
%   [BLEND]     1 to blend with other images being blended, otherwise 0.
%               Default is 0.
%   [OPACITY]   if scalar, then the opacity of the overlay. Otherwise, a
%               4-element vector as follows:
%                   value below which opacity is minimum
%                   value above which opacity is maximum
%                   minimum opacicity (between 0 and 1)
%                   maximum opacicity (between 0 and 1)
%               Default is 1.
%   [OVERLAY]   'i' image or 'o' outline. Default is 'i'.
%   [MASK]      image to mask with. Only voxels with are nonzero in the
%               mask image are kept. Default is no mask.
%
% All blended images are added first, followed by others in the order
% provided.
%
% OPTIONS is a structure containing one or more fields which fine-tune the
% behavior of MULTIVIEW. The valid options, and their defaults are as
% follows:
%
%   LPI             set to 1 if images are stored in LPI order. Normally
%                   MULTIVIEW assumes that images are stored RPI, so the
%                   default value is 0.
%   SHOWCOORDS      set to 1 to display coordinates on each slice. Default
%                   is 1.
%   COORDSCOLOR     3-element RGB vector defining coordinate color. Default
%                   is [1, 1, 1], i.e. white.
%   MARKLEFTRIGHT   set to 1 to mark left and right on images. Default is
%                   1.
%   LEFTRIGHTCOLOR  3-element RGB vector defining color of left/right
%                   markers. Default is [1, 1, 0], i.e. yellow.
%   TOLERANCE       determines how close a point has to be to a slice to
%                   appear on that slice. The units are millimeters.
%                   Default is 5 mm.
%   SLICESPACING    Distance, in voxels, between adjacent panels. Default
%                   is 5.
%   CROP            6-element vector defining how much to crop images by,
%                   measured in voxels. Order is [left, right, posterior,
%                   anterior, inferior, superior].
%   PRINTWIDTH      width of figure to save, in inches; default is 6"
%   POINTRADIUS     radius of points
%   TEXTSIZE        text size for labeling points
%   GRIDLINES       spacing of gridlines; specify 0 for no gridlines
%   GRIDCOLOR       3-element RGB vector defining color of gridlines
%   CLOSEWINDOW     whether to close the window after saving the figure;
%                   default is 0 (don't close, keep it open)
%
% POINTS is an array of points to mark on the image. Each row represents
% one point, and the columns are as follows:
%     Columns 1, 2 and 3: x, y and z coordinates of point
%     Column 4: marker
%       1: circle
%       2: rectangle
%       3: triangle
%       4: diamond
%     Columns 5, 6, 7: fill color (RGB)
%     Column 8: label (number)
%     Columns 9, 10, 11: label color (RGB)
%
% Notes on location and orientation
%
% MULTIVIEW assumes that all images are identical in terms of location and
% orientation. Furthermore, it assumes that images are stored in RPI
% order in files, unless the LPI option is used to specify LPI storage. RPI
% is the typical order used by SPM2, SPM5 and FSL.
%
% The ORIGIN field from the ANALYZE header is used to interpret slice
% coordinates and point coordinates. However, NIFTI location/orientation
% information can also be undertood under very specific circumstances. That
% is, outputs of SPM5 should be read properly. The NIFTI
% location/orientation information in these files is converted to an
% ANALYZE-style origin internally.
%
% There is one case where image sizes do not have to match, and that is
% when functional images with 2mm voxels are overlaid on anatomical images
% with 1mm voxels. This is only possible for standard MNI-sized images.
%
% Version 1.01
% Copyright 2005 - 2014 Stephen M. Wilson <smwilson@u.arizona.edu>
%
% This is free software released under the terms of the GNU General Public
% Licence.


% argument checking
if nargin < 3, error('Not enough input arguments.'); end
if nargin < 4, options = []; end
if nargin < 5, points = []; end

% fill in default options
if ~isfield(options, 'lpi'), options.lpi = 0; end
if ~isfield(options, 'showcoords'), options.showcoords = 1; end
if ~isfield(options, 'coordscolor'), options.coordscolor = [1 1 1]; end
if ~isfield(options, 'markleftright'), options.showleftright = 1; end
if ~isfield(options, 'leftrightcolor'), options.leftrightcolor = [1 1 0]; end
if ~isfield(options, 'tolerance'), options.tolerance = 5; end
if ~isfield(options, 'slicespacing'), options.slicespacing = 5; end % pixels between panels
if ~isfield(options, 'crop'), options.crop = [0 0 0 0 0 0]; end
if ~isfield(options, 'printwidth'), options.printwidth = 6; end
if ~isfield(options, 'pointradius'), options.pointradius = 2; end % voxels
if ~isfield(options, 'textsize'), options.textsize = 6; end % voxels
if ~isfield(options, 'gridlines'), options.gridlines = 20; end
if ~isfield(options, 'gridcolor'), options.gridcolor = [0.5 0.5 0.5]; end
if ~isfield(options, 'closewindow'), options.closewindow = 0; end

% read anatomical image
fprintf('Processing anatomical image %s\n', images(1).fname);
[hdr, img] = read_nifti(images(1).fname);
img(isnan(img)) = 0;

% flip if RPI
if ~options.lpi
  img = flipdim(img, 1);
end

origdim = hdr.dim(2:4);
vox = abs(hdr.pixdim(2:4));
% sometimes vox not quite integer, causes problems
for i = 1:3
  if abs(vox(i) - round(vox(i))) < 0.0001
    vox(i) = round(vox(i));
  end
end

if isfield(hdr, 'origin') & ~isempty(hdr.origin)
  origin = hdr.origin(1:3);
elseif all(origdim == [181 217 181]) && all(vox == [1 1 1])
  warning('Setting origin assuming anatomical is in MNI space');
  origin = [91 127 73];
elseif all(origdim == [91 109 91]) && all(vox == [2 2 2])
  warning('Setting origin assuming anatomical is in MNI space');
  origin = [46 64 37];
else
  warning('Setting origin to middle of the image');
  origin = round(origdim / 2);
end

% crop if requested
img = img((1 + options.crop(1)):(end - options.crop(2)), ...
  (1 + options.crop(3)):(end - options.crop(4)), ...
  (1 + options.crop(5)):(end - options.crop(6)));
dim = size(img);
origin = origin - [options.crop(1) options.crop(3) options.crop(5)];

leftextent = (1 - origin(1)) * vox(1);
rightextent = (dim(1) - origin(1)) * vox(1);
posteriorextent = (1 - origin(2)) * vox(2);
anteriorextent = (dim(2) - origin(2)) * vox(2);
inferiorextent = (1 - origin(3)) * vox(3);
superiorextent = (dim(3) - origin(3)) * vox(3);

% be careful if not cropping symmetrically from left and right, there may be a problem

% unless specified, anatomical min and max determined automatically
if isfield(images, 'scale') && ~isempty(images(1).scale)
  anatmin = images(1).scale(1);
  anatmax = images(1).scale(2);
else
  vals = img(:);
  vals = sort(vals(1:10:end));
  wm = vals(round(length(vals) * 90 / 100));
  f = find(vals > wm / 4, 1);
  vals = vals(f:end);
  anatmin = vals(round(length(vals) * 10 / 100));
  anatmax = vals(round(length(vals) * 90 / 100));
  fprintf('Scaling anatomical to [%.2f, %.2f].\n', anatmin, anatmax);
end

% process slice coordinates
nslices = length(slices);
for s = 1:nslices
  p = slices(s).plane(1);
  if p == 's' || p == 'x' % sagittal
    slices(s).pn = 1;
    slices(s).coordlabel = 'x';
    slices(s).width = dim(2) * vox(2) / min(vox);
    slices(s).height = dim(3) * vox(3) / min(vox);
  elseif p == 'c' || p == 'y' % coronal
    slices(s).pn = 2;
    slices(s).coordlabel = 'y';
    slices(s).width = dim(1) * vox(1) / min(vox);
    slices(s).height = dim(3) * vox(3) / min(vox);
  elseif p == 'h' || p == 'a' || p == 'z' % axial
    slices(s).pn = 3;
    slices(s).coordlabel = 'z';
    slices(s).width = dim(1) * vox(1) / min(vox);
    slices(s).height = dim(2) * vox(2) / min(vox);
  else
    error('Plane not recognized for slice %d.', s);
  end
  
  % determine internal offset
  slices(s).icoord = round(slices(s).coord / vox(slices(s).pn)) + origin(slices(s).pn);
  
  if slices(s).icoord < 1 || slices(s).icoord > dim(slices(s).pn)
    error('Slice %d is outside the bounds of the image.', s);
  end
end

% get anatomical slices
for s = 1:nslices
  if slices(s).pn == 1
    r_{1, s} = img(slices(s).icoord, :, :);
  elseif slices(s).pn == 2
    r_{1, s} = img(:, slices(s).icoord, :);
  elseif slices(s).pn == 3
    r_{1, s} = img(:, :, slices(s).icoord);
  end
  r_{1, s} = squeeze(r_{1, s});
  r_{1, s} = scaleimg(r_{1, s}, anatmin, anatmax, 0.15, 1);
  g_{1, s} = r_{1, s};
  b_{1, s} = r_{1, s};
end

% read functional images
nimages = length(images);
for i = 2:nimages
  fprintf('Processing image %s\n', images(i).fname);
  
  % interpret MATLAB colormaps
  if ischar(images(i).cmap)
    eval(sprintf('images(i).cmap = %s(100);', images(i).cmap));
  end
  
  % fill in blank fields with defaults
  if ~isfield(images, 'blend') || isempty(images(i).blend)
    images(i).blend = 0;
  end
  if ~isfield(images, 'opacity') || isempty(images(i).opacity)
    images(i).opacity = 1;
  end
  if ~isfield(images, 'overlay') || isempty(images(i).overlay)
    images(i).overlay = 'i';
  end
  if ~isfield(images, 'mask') || isempty(images(i).mask)
    images(i).mask = [];
  end
  
  % read the image
  [hdr, img] = read_nifti(images(i).fname);
  img(isnan(img)) = 0;
  
  % old version required min and max, now replaced with 2-vector scale
  if ~isfield(images, 'scale') || isempty(images(i).scale)
    images(i).scale = [-max(abs(img(:))), +max(abs(img(:)))];
  end
  
  % mask if requested
  if ~isempty(images(i).mask)
    [maskhdr, maskimg] = read_nifti(images(i).mask);
    maskimg(isnan(maskimg)) = 0;
    img = img .* (maskimg ~= 0);
  end
  
  % flip if RPI
  if ~options.lpi
    img = flipdim(img, 1);
  end
  
  % check geometry
  if any(hdr.dim(2:4) ~= origdim)
    if all(origdim == [181 217 181]) && all(hdr.dim(2:4) == [91 109 91]) % 2 mm MNI image but 1 mm underlay
      bigimage = zeros([91 109 91] * 2);
      for z = 1:91
        bigimage(:, :, z * 2 - 1) = kron(img(:, :, z), ones(2));
        bigimage(:, :, z * 2) = bigimage(:, :, z * 2 - 1);
      end
      img = bigimage;
      img = img(1:end - 1, 1:end - 1, 1:end - 1);
    elseif all(origdim == [157 189 137]) && all(hdr.dim(2:4) == [79 95 69]) % 2 mm MNI image but 1 mm underlay, smaller SPM bounding boxes
      bigimage = zeros([79 95 69] * 2);
      for z = 1:69
        bigimage(:, :, z * 2 - 1) = kron(img(:, :, z), ones(2));
        bigimage(:, :, z * 2) = bigimage(:, :, z * 2 - 1);
      end
      img = bigimage;
      img = img(1:end - 1, 1:end - 1, 1:end - 1);
    else
      error('The image %s is not in the same space as the anatomical image.', images(i).fname);
    end
  end
  
  % crop if requested
  img = img((1 + options.crop(1)):(end - options.crop(2)), ...
    (1 + options.crop(3)):(end - options.crop(4)), ...
    (1 + options.crop(5)):(end - options.crop(6)));
  
  % get rid of VLSM NaNs
  img(img > 327.66 & img < 327.68) = 0;
  img(isnan(img)) = 0;
  
  % make slices
  for s = 1:nslices
    [r_{i, s}, g_{i, s}, b_{i, s}, o_{i, s}, sd_{i, s}] = ...
      makeslice(slices(s), images(i), img);
  end
end

% calculate the panel locations
fprintf('Assembling figure\n');
maxright = 0;
maxbottom = 0;
for s = 1:nslices
  if s == 1
    left = options.slicespacing;
    top = options.slicespacing;
  elseif slices(s).pos(1) == 'b' % below
    left = slices(slices(s).ref).rect(1);
    top = slices(slices(s).ref).rect(6) + options.slicespacing;
  elseif slices(s).pos(1) == 'r' % to the right of
    left = slices(slices(s).ref).rect(5) + options.slicespacing;
    top = slices(slices(s).ref).rect(2);
  else
    error('The panel location for slice %d is not properly specified.', s);
  end
  slices(s).rect = [left, top, slices(s).width, slices(s).height, ...
    left + slices(s).width, top + slices(s).height]; % left top width height right bottom
  if slices(s).rect(5) > maxright
    maxright = slices(s).rect(5);
  end
  if slices(s).rect(6) > maxbottom
    maxbottom = slices(s).rect(6);
  end
end
figwidth = maxright + options.slicespacing;
figheight = maxbottom + options.slicespacing;

% create figure
screenrect = get(0, 'ScreenSize');
dpi = get(0, 'ScreenPixelsPerInch');
if options.printwidth
  magfactor = options.printwidth * dpi / figwidth;
else
  magfactor = 1;
end
figrect = [50, screenrect(4) - figheight * magfactor - 100, ...
  figwidth * magfactor, figheight * magfactor];
if options.closewindow
  figure('Visible', 'off', 'Units', 'pixels', 'Position', figrect);
else
  figure('Units', 'pixels', 'Position', figrect);
end

% create axes
for s = 1:nslices
  axisrect = [slices(s).rect(1), figheight - slices(s).rect(6), ...
    slices(s).rect(3), slices(s).rect(4)] .* magfactor;
  slices(s).axishdl = axes('Units', 'pixels', 'Position', axisrect);
  set(gca, 'XTick', [], 'YTick', []);
end

% which images are to be blended and which overlaid?
blends = [];
overlays = [];
for i = 2:nimages
  if images(i).blend
    blends = [blends, i];
  else
    overlays = [overlays, i];
  end
end

% combine images
pointsdone = zeros(1, size(points, 1));
for s = 1:nslices
  % start with anatomical
  r = r_{1, s};
  g = g_{1, s};
  b = b_{1, s};
  
  % blends
  if ~isempty(blends)
    count = 0;
    merge = [];
    for i = blends
      count = count + 1;
      rr = r_{i, s};
      gg = g_{i, s};
      bb = b_{i, s};
      opacity = o_{i, s};
      sd = sd_{i, s};
      merge(:, 1, count) = rr(:);
      merge(:, 2, count) = gg(:);
      merge(:, 3, count) = bb(:);
      merge(:, 4, count) = opacity(:);
      merge(:, 5, count) = sd(:);
    end
    for i = 1:count
      merge(:, 6:8, i) = rgb2hsv(merge(:, 1:3, i)); % hsv
      [cart1, cart2] = pol2cart(merge(:, 6, i) * 2 * pi, merge(:, 7, i));
      merge(:, 9, i) = cart1;
      merge(:, 10, i) = cart2;
    end
    opacity = sum(merge(:, 4, :), 3);
    opacity = min(opacity, 1);
    mix = zeros(size(merge, 1), 3);
    for i = 1:count
      mix(:, 1) = mix(:, 1) + merge(:, 9, i) .* merge(:, 5, i);
      mix(:, 2) = mix(:, 2) + merge(:, 10, i) .* merge(:, 5, i);
      mix(:, 3) = mix(:, 3) + merge(:, 8, i) .* merge(:, 5, i);
    end
    mix = mix ./ max(repmat(sum(merge(:, 5, :), 3), [1 3]), 0.00001);
    [pol1, pol2] = cart2pol(mix(:, 1), mix(:, 2));
    hsvmix = [pol1 / (2 * pi), pol2, mix(:, 3)];
    hsvmix(hsvmix(:, 1) < 0, 1) = hsvmix(hsvmix(:, 1) < 0, 1) + 1;
    rgbmix = hsv2rgb(hsvmix);
    rr = reshape(rgbmix(:, 1), size(rr));
    gg = reshape(rgbmix(:, 2), size(gg));
    bb = reshape(rgbmix(:, 3), size(bb));
    opacity = reshape(opacity, size(rr));
    r = (1 - opacity) .* r + opacity .* rr;
    g = (1 - opacity) .* g + opacity .* gg;
    b = (1 - opacity) .* b + opacity .* bb;
  end
  
  % overlays
  for i = overlays
    rr = r_{i, s};
    gg = g_{i, s};
    bb = b_{i, s};
    opacity = o_{i, s};
    r = (1 - opacity) .* r + opacity .* rr;
    g = (1 - opacity) .* g + opacity .* gg;
    b = (1 - opacity) .* b + opacity .* bb;
  end
  
  % put everything together
  finalimg = [];
  finalimg(:, :, 1) = flipud(r');
  finalimg(:, :, 2) = flipud(g');
  finalimg(:, :, 3) = flipud(b');
  finalimg(finalimg > 1) = 1; % just in case there are rounding errors
  finalimg(finalimg < 0) = 0; % just in case there are rounding errors
  
  if slices(s).pn == 1 && slices(s).coord <= 0
    finalimg = flipdim(finalimg, 2);
  end
  axes(slices(s).axishdl);
  if options.closewindow % because axes makes the figure visible for some unknown reason
    set(gcf, 'Visible', 'off');
  end
  image(finalimg);
  set(gca, 'XTick', [], 'YTick', []);
  
  % plot gridlines
  if options.gridlines
    for plane = 1:3
      if slices(s).pn ~= plane
        eccen = 0;
        linesdrawn = 1;
        linewidth = 1;
        while linesdrawn
          linesdrawn = 0;
          for sign = [-1 1]
            coord = sign * eccen;
            lx = [];
            switch(plane)
              case 1
                if coord > leftextent && coord < rightextent
                  lx = repmat(1 + (coord - leftextent) / vox(1), [2 1]);
                  ly = [0; size(finalimg, 1) + 1];
                end
              case 2
                if coord > posteriorextent && coord < anteriorextent
                  if slices(s).pn == 1 && slices(s).coord <= 0
                    lx = repmat(1 + (anteriorextent - coord) / vox(2), [2 1]);
                    ly = [0; size(finalimg, 1) + 1];
                  elseif slices(s).pn == 1 && slices(s).coord > 0
                    lx = repmat(1 + (coord - posteriorextent) / vox(2), [2 1]);
                    ly = [0; size(finalimg, 1) + 1];
                  elseif slices(s).pn == 3
                    lx = [0; size(finalimg, 2) + 1];
                    ly = repmat(1 + (anteriorextent - coord) / vox(2), [2 1]);
                  end
                end
              case 3
                if coord > inferiorextent && coord < superiorextent
                  lx = [0; size(finalimg, 2) + 1];
                  ly = repmat(1 + (superiorextent - coord) / vox(3), [2 1]);
                end
            end
            if ~isempty(lx)
              if options.gridlines > 0
                line(lx, ly, 'Color', options.gridcolor, 'LineWidth', linewidth);
              else
                if lx(1) == lx(2)
                  line(lx, [0 3], 'Color', options.gridcolor, 'LineWidth', linewidth);
                  line(lx, [ly(2) - 3; ly(2)], 'Color', options.gridcolor, 'LineWidth', linewidth);
                else
                  line([0 3], ly, 'Color', options.gridcolor, 'LineWidth', linewidth);
                  line([lx(2) - 3; lx(2)], ly, 'Color', options.gridcolor, 'LineWidth', linewidth);
                end
              end
              linesdrawn = linesdrawn + 1;
            end
          end
          eccen = eccen + abs(options.gridlines);
          linewidth = 0.5;
        end
      end
    end
  end
  
  % plot points
  if ~isempty(points)
    hold on
    for p = 1:size(points, 1)
      pointonslice = 0;
      if options.tolerance > 0
        pointdist = abs(points(p, slices(s).pn) - slices(s).coord);
        if pointdist <= options.tolerance
          pointonslice = 1;
        end
      else
        if points(p, slices(s).pn) >= min(slices(s).pointrange) && ...
            points(p, slices(s).pn) <= max(slices(s).pointrange)
          pointonslice = 1;
        end
      end
      
      if pointonslice
        switch(slices(s).pn)
          case 1
            if points(p, 2) >= posteriorextent && points(p, 2) <= anteriorextent && ...
                points(p, 3) >= inferiorextent && points(p, 3) <= superiorextent
              if slices(s).coord <= 0
                px = 1 + (anteriorextent - points(p, 2)) / vox(2);
              else
                px = 1 + (points(p, 2) - posteriorextent) / vox(2);
              end
              py = 1 + (superiorextent - points(p, 3)) / vox(3);
            else
              px = -1;
            end
          case 2
            if points(p, 1) >= leftextent && points(p, 1) <= rightextent && ...
                points(p, 3) >= inferiorextent && points(p, 3) <= superiorextent
              px = 1 + (points(p, 1) - leftextent) / vox(1);
              py = 1 + (superiorextent - points(p, 3)) / vox(3);
            else
              px = -1;
            end
          case 3
            if points(p, 1) >= leftextent && points(p, 1) <= rightextent && ...
                points(p, 2) >= posteriorextent && points(p, 2) <= anteriorextent
              px = 1 + (points(p, 1) - leftextent) / vox(1);
              py = 1 + (anteriorextent - points(p, 2)) / vox(2);
            else
              px = -1;
            end
        end
        if px >= 0
          rad = options.pointradius;
          switch(points(p, 4))
            case 1 % circle
              rectangle('Position', [px - rad, ...
                py - rad, rad * 2, ...
                rad * 2], 'Curvature', [1 1], ...
                'FaceColor', points(p, 5:7));
            case 2 % rectangle
              rectangle('Position', [px - rad * 0.8, ...
                py - rad * 0.8, rad * 0.9 * 2, ...
                rad * 0.9 * 2], 'FaceColor', points(p, 5:7));
            case 3 % triangle
              fill([px - rad - 0.2, px, px + rad + 0.2], [py + rad - 0.4, py - rad - 0.6, py + rad - 0.4], ...
                points(p, 5:7));
            case 4 % diamond
              fill([px - rad * 1.2, px, px + rad * 1.2, px], [py, py - rad * 1.2, py, py + rad * 1.2], ...
                points(p, 5:7));
          end
          if points(p, 8) <= 9
            fontsize = options.textsize;
          else
            fontsize = options.textsize * 0.75;
          end
          if points(p, 8) < 32
            label = sprintf('%d', points(p, 8));
          else
            label = sprintf('%c', points(p, 8));
          end
          text(px, py, label, 'HorizontalAlignment', 'center', ...
            'FontName', 'Arial', 'FontSize', fontsize, 'Color', points(p, 9:11));
          pointsdone(p) = 1; % mark as done
        end
      end
    end
  end
  
  % plot coordinates
  if options.showcoords
    set(gca, 'Units', 'points');
    axispos = get(gca, 'Position');
    if slices(s).coord < 0
      minussign = '-';
    else
      minussign = '';
    end
    coordstring = sprintf('%s = %s%d', slices(s).coordlabel, minussign, abs(slices(s).coord));
    text(axispos(3) - 4, axispos(4) - 6, coordstring, 'Units', 'points', ...
      'HorizontalAlignment', 'right', ...
      'FontName', 'Arial', 'FontSize', 9, 'Color', options.coordscolor);
  end
  if ~isempty(outfname)
    imwrite(finalimg, sprintf('%s_slice%02d.tif', outfname, s), 'TIFF', 'Compression', 'lzw');
  end
end

% caption
% axisrect = [0, 0, 100, (nimages - 1) * 20];
% captionaxishdl = axes('Units', 'pixels', 'Position', axisrect);
% set(gca, 'XTick', [], 'YTick', []);
% for i = 2:nimages
%     % color bar or line
%     if image(i).overlay(1) == 'i'
%         axisrect = [0, (nimages - i) * 20, 50, 20];
%         axes('Units', 'pixels', 'Position', axisrect);
%         if size(image(i).cmap, 1) == 1
%             cmap = ;
%         else
%             cmap = image(i).cmap;
%         end
%         set(gca, 'XTick', [], 'YTick', []);
%     else
%     end
%
%     % label
%     if isfield(images, 'caption') & ~isempty(images(i).caption)
%         label = images(i).caption;
%     else
%         [path, label] = fileparts(images(i).fname);
%     end
%     h = text(50, (nimages - i + 1) * 20, label, 'Units', 'pixels', 'FontName', 'Arial', 'FontSize', 9, ...
%         'Interpreter', 'none', 'VerticalAlignment', 'Cap');
% end

% save to output file
if ~isempty(outfname)
  ext = outfname((end - 2):end);
  fprintf('Writing figure to file\n');
  set(gcf, 'PaperPositionMode', 'auto'); % print at screen size
  if strcmp(ext, 'jpg')
    print('-djpeg', '-r0', outfname);
  elseif strcmp(ext, 'eps')
    print('-depsc', '-r0', outfname);
  elseif strcmp(ext, 'tif')
    print('-dtiff', '-r0', outfname);
    %     elseif strcmp(ext, 'tif')
    %         imwrite(finalimg, [outfname '.tif'], 'TIFF', 'Compression', 'lzw');
  else
    print('-depsc', '-r0', [outfname '.eps']);
    print('-djpeg', '-r0', [outfname '.jpg']);
  end
  fprintf('Done.\n');
end

% close the window after saving, if requested
if options.closewindow
  close(gcf);
end

% check for points which were missed
pointsmissed = 0;
for p = 1:size(points, 1)
  if pointsdone(p) == 0
    if ~pointsmissed
      fprintf('The following points were not displayed on any slice:\n');
      pointsmissed = 1;
    end
    fprintf('%d: ', p);
    fprintf('%.1f ', points(p, :));
    fprintf('\n');
  end
end

function [r, g, b, o, sd] = makeslice(slice, image, img)

% returns r, g and b maps as well as opacity for a given slice

% get slice
if slice.pn == 1
  d = img(slice.icoord, :, :);
elseif slice.pn == 2
  d = img(:, slice.icoord, :);
elseif slice.pn == 3
  d = img(:, :, slice.icoord);
end
d = squeeze(d);

opac = image.opacity;
if isempty(opac)
  opac = 1;
end

if image.overlay(1) == 'i' % color image
  % scale
  sd = scaleimg(d, image.scale(1), image.scale(2), 0, 1);
  
  % calculate opacity
  if length(opac) == 4
    o = scaleimg(d, image.opacity(1), image.opacity(2), image.opacity(3), image.opacity(4));
    o = o .* (sd ~= 0);
  else
    o = opac .* (sd ~= 0);
    o = o .* (sd ~= 0.5);
  end
  
  if size(image.cmap, 1) == 1 % single color continuum
    r = sd * image.cmap(1);
    g = sd * image.cmap(2);
    b = sd * image.cmap(3);
    
  else % colormap
    reds = [0; image.cmap(:, 1)];
    r = reds(ceil(sd * size(image.cmap, 1)) + 1);
    greens = [0; image.cmap(:, 2)];
    g = greens(ceil(sd * size(image.cmap, 1)) + 1);
    blues = [0; image.cmap(:, 3)];
    b = blues(ceil(sd * size(image.cmap, 1)) + 1);
  end
  
elseif image.overlay(1) == 'o' % outline
  if image.scale(1) ~= 0
    sd = (d >= image.scale(1));
  else
    sd = (d > image.scale(1)); % otherwise 0 background will cause problems
  end
  shiftlist = [1 0; -1 0; 0 1; 0 -1];
  sumimg = sd;
  for i = 1:size(shiftlist, 1)
    shiftimg = sd;
    for plane = 1:2
      if shiftlist(i, plane) > 0
        shiftimg = [zeros(size(shiftimg, 1), shiftlist(i, plane)), shiftimg(:, 1:(end - shiftlist(i, plane)))];
      elseif shiftlist(i, plane) < 0
        shiftimg = [shiftimg(:, (1 - shiftlist(i, plane)):end), zeros(size(shiftimg, 1), -shiftlist(i, plane))];
      end
      shiftimg = shiftimg';
    end
    sumimg = sumimg + shiftimg;
  end
  sumimg(sumimg >= 2) = 0;
  sumimg(sumimg ~= 0) = 1;
  
  r = sumimg * image.cmap(1);
  g = sumimg * image.cmap(2);
  b = sumimg * image.cmap(3);
  o = sumimg * opac;
  sd = sumimg;
end


function sd = scaleimg(img, imin, imax, omin, omax)

% scales an image such that values between imin and imax now lie between
% omin and omax. values above and below are clipped.

sd = omin + (img - imin) / (imax - imin) * (omax - omin);
sd(sd < omin) = omin;
sd(sd > omax) = omax;
